/*
 * Created on 13/lug/06
 *
 */
package portachiavi;

import java.io.File;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import java.util.Set;

import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;

import portachiavi.exception.EntryNotFoundException;
import portachiavi.exception.KeyLoadingException;
import portachiavi.exception.KeySavingException;
import portachiavi.gui.DoublePasswordDialog;
import portachiavi.gui.EntryDialog;
import portachiavi.gui.PasswordDialog;
import portachiavi.gui.ShowEntryDialog;

/**
 * Controllore del portachiavi
 * 
 * @author andrea
 * 
 */
public class Controller implements IController {

	private JFileChooser jfc = null;

	private final String KEY_FILENAME = ".portachiavi.key";

	private File keyFile, objFile;

	private JList listViewer;

	private boolean modified;

	private char[] password;

	private IPortachiavi portachiavi;

	private final String PORTACHIAVI_FILENAME = ".portachiavi.obj";

	private final String USERDIR = System.getProperty("user.home");

	/**
	 * Costruttore.
	 * 
	 * @param portachiavi
	 *            il portachiavi su cui lavorare
	 * @param output
	 *            la lavagna per comunicare con l'utente
	 */
	public Controller(IPortachiavi portachiavi, JList listViewer) {
		this.portachiavi = portachiavi;
		this.listViewer = listViewer;
		objFile = new File(USERDIR, PORTACHIAVI_FILENAME);
		keyFile = new File(USERDIR, KEY_FILENAME);
		jfc = new JFileChooser(USERDIR);
		autoLoadPortachiavi();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#addEntry()
	 */
	public boolean addEntry() {
		EntryDialog ed = new EntryDialog();
		ed.setVisible(true);
		String id = ed.getId();
		String username = ed.getUsername();
		String password = ed.getPassword();
		if (id.length() != 0) {
			if (portachiavi.addEntry(id, new LoginData(username, password))) {
				printEntryList();
				modified = true;
				return true;
			} else {
				JOptionPane.showMessageDialog(null,
						"[ADD] Entry gia' presente!", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				return false;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#autoLoadPortachiavi()
	 */
	public void autoLoadPortachiavi() {
		if (objFile.exists() && keyFile.exists()) {
			do {
				setPassword();
			} while (!loadPortachiavi());
		} else {
			setFirstPassword();
			savePortachiavi();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#exit()
	 */
	public void exit() {
		if (modified) {
			int r = JOptionPane.showConfirmDialog(null,
					"Vuoi salvare prima di uscire?", "DOMANDA",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.WARNING_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/warning.png")));
			if (r == JOptionPane.YES_OPTION) {
				if (savePortachiavi()) {
					System.exit(0);
				}
			} else if (r == JOptionPane.NO_OPTION) {
				System.exit(0);
			} else if (r == JOptionPane.CANCEL_OPTION) {
				return;
			}
		} else {
			System.exit(0);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#export2TextFile()
	 */
	public boolean export2TextFile() {
		JOptionPane
				.showMessageDialog(
						null,
						"Il file di testo su cui viene esportato il\nportachiavi non sara' cifrato!\nEsportate a vostro rischio e pericolo!",
						"ATTENZIONE!", JOptionPane.INFORMATION_MESSAGE,
						new ImageIcon(getClass().getResource(
								"/data/warning.png")));
		int r = jfc.showSaveDialog(null);
		if (r == JFileChooser.APPROVE_OPTION) {
			File textFile = jfc.getSelectedFile();
			try {
				portachiavi.export2TextFile(textFile);
				return true;
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null,
						"[EXPORT] Errore di scrittura sul file.", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
				return false;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#loadPortachiavi()
	 */
	public boolean loadPortachiavi() {
		try {
			portachiavi.loadList(password, objFile, keyFile);
			printEntryList();
			return true;
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (KeyLoadingException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null,
					"La password inserita non e' corretta. Riprova.", "ERRORE",
					JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/error.png")));
		}
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#modificaEntry()
	 */
	public boolean modificaEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			try {
				ILoginData auth = portachiavi.searchEntry(id);
				EntryDialog ed = new EntryDialog(id, auth.getUser(), auth
						.getPassword());
				ed.setVisible(true);
				portachiavi.removeEntry(id);
				portachiavi.addEntry(ed.getId(), new LoginData(
						ed.getUsername(), ed.getPassword()));
				printEntryList();
				modified = true;
				return true;
			} catch (EntryNotFoundException e) {
				JOptionPane.showMessageDialog(null, "[MODIFY] Entry "
						+ e.getMessage() + " non trovata", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[MODIFY] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#printEntryList()
	 */
	public void printEntryList() {
		Set<String> a = portachiavi.getEntryList();
		Iterator<String> i = a.iterator();
		String[] list = new String[a.size()];
		int j = 0;
		while (i.hasNext()) {
			list[j] = i.next();
			j++;
		}
		listViewer.setListData(list);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#removeEntry()
	 */
	public boolean removeEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			int res = JOptionPane.showConfirmDialog(null,
					"Eliminare definitivamente la riga \"" + id + "\"?",
					"DOMANDA", JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/warning.png")));
			if (res == JOptionPane.YES_OPTION) {
				try {
					portachiavi.removeEntry(id);
					printEntryList();
					modified = true;
					return true;
				} catch (EntryNotFoundException e) {
					JOptionPane.showMessageDialog(null, "[REMOVE] Entry "
							+ e.getMessage() + " non trovata", "ERRORE",
							JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
									.getResource("/data/error.png")));
					e.printStackTrace();
				}
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[REMOVE] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#savePortachiavi()
	 */
	public boolean savePortachiavi() {
		try {
			portachiavi.saveList(password, objFile, keyFile);
			modified = false;
			return true;
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (NoSuchPaddingException e) {
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (KeySavingException e) {
			e.printStackTrace();
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#searchEntry()
	 */
	public boolean searchEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			try {
				ILoginData auth = portachiavi.searchEntry(id);
				ShowEntryDialog sed = new ShowEntryDialog(id, auth.getUser(),
						auth.getPassword());
				sed.setVisible(true);
				return true;
			} catch (EntryNotFoundException e) {
				JOptionPane.showMessageDialog(null, "[SEARCH] Entry "
						+ e.getMessage() + " non trovata", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[SEARCH] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#setFirstPassword()
	 */
	public void setFirstPassword() {
		DoublePasswordDialog pd = new DoublePasswordDialog();
		pd.setVisible(true);
		password = pd.getPassword();
		if (password == null) {
			System.exit(0);
		} else {
			modified = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#setPassword()
	 */
	public void setPassword() {
		PasswordDialog pd = new PasswordDialog();
		pd.setVisible(true);
		password = pd.getPassword();
		if (password == null) {
			System.exit(0);
		}
	}

}
