/*
 * Created on 13/lug/06
 *
 */
package portachiavi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SealedObject;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

import portachiavi.exception.EntryNotFoundException;
import portachiavi.exception.KeyLoadingException;
import portachiavi.exception.KeySavingException;

public class Portachiavi implements Serializable, IPortachiavi {

	private static final long serialVersionUID = 1L;

	private final int ITERATIONS = 1000;

	private HashMap<String, ILoginData> list = null;

	/**
	 * Costruttore
	 * 
	 */
	public Portachiavi() {
		list = new HashMap<String, ILoginData>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#addEntry(java.lang.String,
	 * portachiavi.ILoginData)
	 */
	public boolean addEntry(String id, ILoginData auth) {
		if (list.containsKey(id)) {
			return false;
		} else {
			list.put(id, auth);
			return true;
		}
	}

	/**
	 * 
	 * @param password
	 * @param input
	 * @return
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidKeySpecException
	 * @throws NoSuchPaddingException
	 * @throws InvalidKeyException
	 * @throws InvalidAlgorithmParameterException
	 */
	private Key decryptKey(char[] password, byte[] input)
			throws NoSuchAlgorithmException, InvalidKeySpecException,
			NoSuchPaddingException, InvalidKeyException,
			InvalidAlgorithmParameterException {

		// recupero il salt e la chiave cifrata dall'array in input
		byte salt[] = new byte[8];
		byte keyEncrypted[] = new byte[input.length - 8];
		for (int i = 0; i < 8; i++) {
			salt[i] = input[i];
		}
		for (int i = 8; i < input.length; i++) {
			keyEncrypted[i - 8] = input[i];
		}

		// definisco le specifiche per la creazione della chiave dalla password
		PBEKeySpec keySpec = new PBEKeySpec(password);
		SecretKeyFactory keyFactory = SecretKeyFactory
				.getInstance("PBEWithMD5AndDES");
		SecretKey PBEKey = keyFactory.generateSecret(keySpec);
		PBEParameterSpec paramSpec = new PBEParameterSpec(salt, ITERATIONS);
		Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");

		// inizializzo il cifratore e decifro la chiave
		cipher.init(Cipher.UNWRAP_MODE, PBEKey, paramSpec);
		Key TDESKey = cipher.unwrap(keyEncrypted, "TripleDES",
				Cipher.SECRET_KEY);
		return TDESKey;
	}

	/**
	 * 
	 * @param password
	 * @param TDESKey
	 * @return
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidKeySpecException
	 * @throws NoSuchPaddingException
	 * @throws InvalidKeyException
	 * @throws InvalidAlgorithmParameterException
	 * @throws IllegalBlockSizeException
	 */
	private byte[] encryptKey(char[] password, Key TDESKey)
			throws NoSuchAlgorithmException, InvalidKeySpecException,
			NoSuchPaddingException, InvalidKeyException,
			InvalidAlgorithmParameterException, IllegalBlockSizeException {

		// genero il salt: una combinazione casuale di bytes
		byte[] salt = new byte[8];
		SecureRandom random = new SecureRandom();
		random.nextBytes(salt);

		// definisco le specifiche per la creazione della chiave dalla password
		PBEKeySpec keySpec = new PBEKeySpec(password);
		SecretKeyFactory keyFactory = SecretKeyFactory
				.getInstance("PBEWithMD5AndDES");
		SecretKey PBEKey = keyFactory.generateSecret(keySpec);
		PBEParameterSpec paramSpec = new PBEParameterSpec(salt, ITERATIONS);
		Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");

		// inizializzo il cifratore e cifro la chiave
		cipher.init(Cipher.WRAP_MODE, PBEKey, paramSpec);
		byte[] keyEncrypted = cipher.wrap(TDESKey);

		// metto nell'array di byte il salt e la chiave cifrata.
		int length = keyEncrypted.length + 8;
		byte result[] = new byte[length];
		for (int i = 0; i < 8; i++) {
			result[i] = salt[i];
		}
		for (int i = 8; i < length; i++) {
			result[i] = keyEncrypted[i - 8];
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#export2TextFile(java.io.File)
	 */
	public void export2TextFile(File textFile) throws IOException {
		String s = "";
		Set<String> idList = list.keySet();
		Iterator<String> i = idList.iterator();
		while (i.hasNext()) {
			String id = i.next();
			ILoginData d = list.get(id);
			s += id + " :\n\t" + d.getUser() + " ;\n\t" + d.getPassword()
					+ " ;\n\n";
		}
		FileWriter fw = new FileWriter(textFile);
		fw.write(s);
		fw.close();
	}

	/**
	 * Genero una nuova chiave per cifrare la lista
	 * 
	 * @return
	 * @throws NoSuchAlgorithmException
	 */
	private Key generateKey() throws NoSuchAlgorithmException {
		KeyGenerator keyGenerator = KeyGenerator.getInstance("TripleDES");
		keyGenerator.init(168);
		return keyGenerator.generateKey();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#getEntryList()
	 */
	public Set<String> getEntryList() {
		return list.keySet();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#getList()
	 */
	public HashMap<String, ILoginData> getList() {
		return list;
	}

	private byte[] loadEncryptedKey(File keyFile) throws IOException {
		byte[] array = new byte[100];
		FileInputStream fis = new FileInputStream(keyFile);
		int n = fis.read(array);
		byte[] output = new byte[n];
		for (int i = 0; i < n; i++) {
			output[i] = array[i];
		}
		fis.close();
		return output;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#loadList(char[])
	 */
	public void loadList(char[] password, File objFile, File keyFile)
			throws IOException, ClassNotFoundException, InvalidKeyException,
			NoSuchAlgorithmException, KeyLoadingException {

		// carico da file l'oggetto sigillato
		FileInputStream fis = new FileInputStream(objFile);
		ObjectInputStream ois = new ObjectInputStream(fis);
		SealedObject so = (SealedObject) ois.readObject();
		ois.close();

		// recupero la chiave per la decifratura
		Key TDESKey = null;
		try {
			TDESKey = decryptKey(password, loadEncryptedKey(keyFile));
		} catch (Exception e) {
			// e.printStackTrace();
			throw new KeyLoadingException(e.getMessage());
		}

		// recupero la lista dall'oggetto sigillato
		list = (HashMap<String, ILoginData>) so.getObject(TDESKey);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#removeEntry(java.lang.String)
	 */
	public void removeEntry(String id) throws EntryNotFoundException {
		if (list.remove(id) == null) {
			throw new EntryNotFoundException(id);
		}
	}

	/**
	 * 
	 * @param array
	 * @param keyFile
	 * @return
	 * @throws IOException
	 */
	private boolean saveEncryptedKey(byte[] array, File keyFile)
			throws IOException {
		FileOutputStream fos = new FileOutputStream(keyFile);
		fos.write(array);
		fos.close();
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#saveList(char[])
	 */
	public void saveList(char[] password, File objFile, File keyFile)
			throws NoSuchAlgorithmException, NoSuchPaddingException,
			InvalidKeyException, IllegalBlockSizeException, IOException,
			KeySavingException {

		// creo una nuova chiave per la lista
		Key TDESKey = generateKey();

		// creo il cifratore della lista
		Cipher cipher = Cipher.getInstance("TripleDES/ECB/PKCS5Padding");
		cipher.init(Cipher.ENCRYPT_MODE, TDESKey);

		// creo l'oggetto sigillato formato dalla lista e dal cifratore.
		SealedObject so = new SealedObject(list, cipher);

		// salvo l'oggetto sigillato
		FileOutputStream fos = new FileOutputStream(objFile);
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(so);
		oos.close();

		// cifro e salvo la chiave
		try {
			saveEncryptedKey(encryptKey(password, TDESKey), keyFile);
		} catch (Exception e) {
			e.printStackTrace();
			throw new KeySavingException(e.getMessage());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IPortachiavi#searchEntry(java.lang.String)
	 */
	public ILoginData searchEntry(String id) throws EntryNotFoundException {
		ILoginData auth = list.get(id);
		if (auth == null) {
			throw new EntryNotFoundException(id);
		} else {
			return auth;
		}
	}
}
